import java.awt.Color;
import java.awt.Font;
import javax.vecmath.*;
import java.io.File;
import java.util.Date;
import edu.cornell.lassp.houle.RngPack.*;

import javax.vecmath.Color3f;

public class Env {
//  **** psuedo-random number generator (PRNG) ****
	static final RanMT mtRNG = new RanMT(new Date());
//  **** Useful Strings ***
	static final String kOnUnits = " /microMolar-s";
	static final String kOffUnits = " /s";
	static final String concUnits = " microMolar";
	static final String distUnits = " microns";
	static final String degUnits = " degrees";
	
//  **** Physical Constants ****
	static final double Boltz = 1.380662e-23;			// Boltzman's constant in J/K
	static final double tempK = 298.15;				// 25 deg Celsius in Kelvin
	static final double AvogadroNum = 6.02214199e23;	// Avogadro's number.. #/Mole
	
//	*** The Virtual World Dimensions ****
	static final Pt3D worldDimension = new Pt3D(1,1,1);
	static final Point3d farfarAway = new Point3d(1000,1000,1000);
	
//  *** Max Things in the World ****
	static final int maxThings = 50000;	// maximum number of Things
	
//	**** Times and Counters
	static private final double deltaT_init = 0.00001;		// seconds
	static double simulationTime = 0;			// keeps track of current time
	static private double runTime_init = 100.0;				// final time to run to
	static int counter = 0;					// keeps track of current integration step number
	
//  **** Files and Paths ****
	static String outFileName = null;
	static String outFolderPath = null;
	static String srcFilePath = null;
	static File paramFile = null;
	
	static final Parameter deltaT = new Parameter ("deltaT"," Time Step", deltaT_init, "seconds");
	static final Parameter runTime = new Parameter ("runTime"," Run Time", runTime_init, "seconds");
	
//  **** For Synchronization of Methods ****
	static Object safeO = new Object();		// handle for operations that cannot be performed at the same time
	
//  **** Flags and Counters for Graphics ****
	static boolean remote = false;			// flag for no graphics instantiation
	static boolean biochemOnly = false;		// flag for only biochemical simulation... no collisions, forces
	static boolean logFiles = false;			// write info to files
	static boolean paintOn = true;
	static boolean toFile = false;
	static boolean paused = true;
	static boolean viewRotation = false;
	
	
	static private final int drawInterval_init = 100;
	static private final int toFileInterval_init = 100;
	static private final double jpegQuality_init = 1.0;
	static private final int remoteReportInterval_init = 1000;		//time-steps
	static final boolean timeStampJPEGs = true;		// write time and other info on JPEG output
	
	static final Parameter drawInterval = new Parameter ("drawInterval"," Drawing Interval", drawInterval_init, "time steps",Parameter.INT);
	static final Parameter toFileInterval = new Parameter ("toFileInterval"," JPEG Interval", toFileInterval_init, "time steps",Parameter.INT);
	static final Parameter jpegQuality = new Parameter ("jpegQuality"," JPEG Quality", jpegQuality_init, " (1.0 is best)");
	static final Parameter remoteReportInterval = new Parameter ("remoteWriteInterval", " Remote Reporting Interval",remoteReportInterval_init,"time steps",Parameter.INT);
	
	static String toFilePath;
	static String toFileName;
	
//  **** Flags for Testing ****
	static boolean sphereMotionOff = false;
	
//  **** Graphics Sizes Etc ****
	static private final int frameWidth_init = 640;
	static private final int frameHeight_init = 480;
	static boolean allowDragResize = false;
	
	static final Parameter frameWidth = new Parameter ("frameWidth"," Frame Width", frameWidth_init, "Pixels",Parameter.INT);
	static final Parameter frameHeight = new Parameter ("frameHeight"," Frame Height", frameHeight_init, "Pixels",Parameter.INT);
	
	static final int plasmidTessalation = 24;
	
	static boolean showBounds = true;
	
	static final double transScale = 8;

//  **** The Simulation Bounds ****
	static private final double boundsLength_init = 1.0;		// (microns) length from tip to tip
	static private final double boundsRadius_init = 0.5;		// (microns) radius of cylinder and hemispherical caps
	static private final double aeta_init = 0.1;			// viscosity in sim bounds.... Pa-s (1e-3 is water)
	static private final double factorXConc_init = 15;			// M concentration of factorX
	static private final double factorYConc_init = 15;			// M concentration of factorY
	
	static final Parameter boundsLength = new Parameter ("boundsLength"," Bounds Length", boundsLength_init, distUnits);
	static final Parameter boundsRadius = new Parameter ("boundsRadius"," Bounds Radius", boundsRadius_init, distUnits);
	
	static private final double boundsGrowthRate_init = 0;

//  ***** Environment inside bounds
	static final Parameter factorXConc = new Parameter ("factorXConc"," FactorX Concentration", factorXConc_init, concUnits, Parameter.DOUBLE,Parameter.CHANGE_VALUE);
	static final Parameter factorYConc = new Parameter ("factorYConc"," FactorY Concentration", factorYConc_init, concUnits, Parameter.DOUBLE,Parameter.CHANGE_VALUE);
	static final Parameter aeta = new Parameter ("aeta"," Viscosity", aeta_init, "Pa-s (1e-3 is water)");
	
//	***** Generic SphereThing ****
	static final double sphereThingRadius = 0.1;	// microns
	
//  ***** Gluttons ****
	static private final int initialGluttons_init = 2;
	static final Parameter initialGluttons = new Parameter ("initialGluttons"," Initial Gluttons", initialGluttons_init," ",Parameter.INT);
	
	static private final double gluttonRadius_init = 0.1;	// microns
	static final Parameter gluttonRadius = new Parameter ("gluttonRadius"," Glutton Radius", gluttonRadius_init, distUnits);
	
	static private final double gluttonMetabolicRate_init = 0.9; // fractional size decrease / sec
	static final Parameter gluttonMetabolicRate = new Parameter ("gluttonMetabolicRate"," Glutton Metabolic Rate", gluttonMetabolicRate_init, " body fraction/sec");

	static private final double gluttonWitherSize_init = 0.001; // microns
	static final Parameter gluttonWitherSize = new Parameter ("gluttonWitherSize"," Glutton Wither Size", gluttonWitherSize_init, distUnits);
	
	static private final double gluttonBurstSize_init = Env.boundsRadius_init; // microns
	static final Parameter gluttonBurstSize = new Parameter ("gluttonBurstSize"," Glutton Burst Size", gluttonBurstSize_init, distUnits);
	
	//	***** Muttons *****
	static private final int initialMuttons_init = 2;
	static final Parameter initialMuttons = new Parameter ("initialMuttons"," Initial Muttons", initialMuttons_init," ",Parameter.INT);
	
	static private final double muttonRadius_init = 0.02;	// microns
	static final Parameter muttonRadius = new Parameter ("muttonRadius"," Mutton Radius", muttonRadius_init, distUnits);
	
	static private final double muttonSpawnRate_init = 12;		// muttons/sec
	static final Parameter muttonSpawnRate = new Parameter ("muttonSpawnRate"," Mutton Spawn Rate", muttonSpawnRate_init, " muttons/sec");

	//	***** 3D Quality *****
	static int graphicsQuality = 0;	// specify graphics quality
	static final int LOW_QUALITY = 0;
	static final int	MID_QUALITY = 1;
	static final int HIGH_QUALITY = 2;
	
	static final int lowSphereTessalation = 10;	// number of polygons to define a sphere
	static final int midSphereTessalation = 18;
	static final int highSphereTessalation = 24;
	
	static final int lowCylXTessalation = 10;		// x and y resolution for cylinders
	static final int lowCylYTessalation = 10;
	static final int midCylXTessalation = 18;
	static final int midCylYTessalation = 18;
	static final int highCylXTessalation = 24;
	static final int highCylYTessalation = 24;
	
	// filament appearance
	static boolean filamentLines = true;
	static boolean filamentCyls = false;
	
	// monomer appearance
	static boolean eachMonomerASphere = false;
	static boolean eachMonomerADot = false;
	
//	 **** GUI AND COLOR RELATED ****
	static final Color3f universeColor3f = new Color3f(0.5f,0.5f,0.5f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f headlightColor = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f cellColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Color3f controlBackColor3f = new Color3f(0.5f,0.5f,0.5f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = Color.BLACK;//Color.WHITE;
	static final Color controlFlashColor = Color.WHITE;
	
	public static void resetTimeDependencies () {

	}
}
