import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Timer;
import java.util.*;

//import FileOps.SourceFileFilter;

public class Sim3D {
	
	// graphics component
	static Sim3DGraphics simGraphics;
	static TimeLoop timeLoop;
	static Random prng = new Random();
	
	
	public Sim3D (String[] args) {
		parseArgs(args);
		
		if (!Env.remote) { 
			simGraphics = new Sim3DGraphics( );
			simGraphics.initialize( );
			simGraphics.buildUniverse( );
			simGraphics.showFrame( ); 
		}
		
		// make Things
		SimBounds.makeInitialBounds();
		Glutton.makeInitialGluttons();
		Mutton.makeInitialMuttons();
		
		// make the time loop instance
		timeLoop = new TimeLoop();
		timeLoop.start();
	}
	
	protected void parseArgs( String[] args ) {
		for ( int i = 0; i < args.length; i++ ) {
			if (args[i].equals("-help")) {
				talkln (" Command line options for ParMSpindle:");
				talkln (" -help: print this message");
				talkln (" -r  : run the code 'remotely', without any graphics output");
				talkln (" -pf -paramfile -paramFile (file) : load parameter values from specified file");
				talkln (" -lf -logfile -logFile (directory) : create specified directory and save all output files to there");
				talkln (" -biochem : run the simulation without any collisions, forces, or brownian motion");
				System.exit(0);
			}
			
			if ( args[i].equals( "-r" )) {
				Env.remote = true;
				Env.paused = false;
			}
			
			if ((args[i].equals("-paramfile")) | (args[i].equals("-pf")) | (args[i].equals("-paramFile"))) {
				File paramFile = new File(args[i+1]);
				if (paramFile.exists()) {
					Env.paramFile = paramFile;
				} else {
					talkln ("Specified param file can't be found... check path, etc");
					System.exit(0);
				}
			}
			
			if (args[i].equals("-biochem")) {
				Env.biochemOnly = true;
			}
			
			
			if ((args[i].equals("-logfile")) | (args[i].equals("-lf")) | (args[i].equals("-logFile"))) {
				File outFolder = new File(args[i+1]);
				File altOutFolder = new File (args[i+1]);
				int j=1;
				while (altOutFolder.isDirectory()) {
					System.out.println (altOutFolder.getName() + " exists.... keeping file name BUT changing directory name");
					altOutFolder = new File(args[i+1] + "." + String.valueOf(j));
					j++;
				}
				altOutFolder.mkdir();
				Env.outFileName = outFolder.getName();
				Env.outFolderPath = altOutFolder.getAbsolutePath();
				// make a directory for the source files
				File srcdir = new File (altOutFolder.getAbsolutePath() + File.separator + Env.outFileName + ".SRC");
				//srcdir.mkdir();
				Env.srcFilePath = srcdir.getAbsolutePath();
				//*******
				Env.logFiles = true;
			}
		}
	}
	
	//
	//  Main
	//
	public static void main( String[] args )
	{
		new Sim3D(args);
		
	}
	
	class TimeLoop extends Thread {
	
		public void run() {
			if (!Env.remote) {
				//parMGraphics.setTopLabels();
				simGraphics.updateBugScene();		// draw once at the beginning of the simulation
				startUpdateTimer();
			}
			if (Env.paramFile != null) { FileOps.loadParamConfig(Env.paramFile); }
			if (Env.outFileName != null) { FileOps.remoteParamConfigSave(); }
			doLoop();
			System.exit(0);
		}
	}
	
	public static void doLoop() {
		boolean paintedThisStep;
		double lastReportTime = System.currentTimeMillis();
		int drawCounter = 0;
		int toFileCounter = 0;
		int remoteOutCounter = 0;
		int qkCounter = 0;
		
		while (Env.simulationTime <= Env.runTime.getValue()) {
			if (Env.paused) { 
				if (Env.viewRotation) { 
					simGraphics.rotateView();  // let's us set up a good looking rotation before starting movie
					try { Thread.sleep(20); } catch (InterruptedException e) { talkln ("error sleeping"); }
				} else {
					try { Thread.sleep(200); } catch (InterruptedException e) { talkln ("error sleeping"); }
				}
			}
			if (!Env.paused) {
					synchronized(Env.safeO) {
					Glutton.checkMuttonCollision();
					for (int i=0;i<Thing.thingCt;i++) {
						Thing.theThings[i].step();
					}
					Mutton.spawnMuttons();
					
					//update counters and flags
					paintedThisStep = false;
					Env.counter++;
					Env.simulationTime += Env.deltaT.getValue();
					remoteOutCounter++;
					qkCounter++;
					
					if (!Env.remote) { 
						//parMGraphics.setTopLabels();
						drawCounter++;
						toFileCounter++;
						
						if ((Env.paintOn) & (drawCounter >= Env.drawInterval.getIntValue())) {
							if (Env.viewRotation) { simGraphics.rotateView(); }
							simGraphics.updateBugScene();
							paintedThisStep = true;
							drawCounter = 0;
						}
						
						if ((Env.toFile) & (toFileCounter >= Env.toFileInterval.getIntValue())) {
							if (!paintedThisStep) {
								if (Env.viewRotation) { simGraphics.rotateView(); }
								simGraphics.updateBugScene();
								paintedThisStep = true;
							}
							simGraphics.captureImage(simGraphics.theCanvas);
							simGraphics.waitOnCanvas(simGraphics.theCanvas);
							toFileCounter = 0;
						}
					
					} else {
						if (remoteOutCounter > Env.remoteReportInterval.getValue()) { 
							double curTime = System.currentTimeMillis();
							double timeTween = curTime - lastReportTime;
							lastReportTime = curTime;
							talk ("Time: " + String.valueOf(Env.simulationTime));
							talk (" , timeTween: " + String.valueOf(timeTween));
							talk (" , ParM Concentration: " + String.valueOf(Env.factorXConc.getValue()));
							remoteOutCounter = 0;
						}
					}
					
					// remove Things AFTER the graphical update... can detach graphics objects of dead things this way
					Thing.removeDeadThings();		// get rid of dead things
					// stochastically create new filaments
				}
			}
		}
	}
	
	public static void startUpdateTimer() {
		int delay = 1000; //milliseconds
		ActionListener taskPerformer = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				simGraphics.ratesControl.updateInfoFields();
				simGraphics.initControl.updateInfoFields();
				simGraphics.graphicsControl.updateInfoFields();
		    }
		};
		new Timer(delay, taskPerformer).start();
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.print(info);
	}
}
